package MT::Plugin::DesignPermissions;
use strict;
use MT;
use MT::Plugin;
use base qw( MT::Plugin );

@MT::Plugin::DesignPermissions::ISA = qw( MT::Plugin );

our $VERSION = '1.0.2';

my $plugin = new MT::Plugin::DesignPermissions( {
    name => 'DesignPermissions',
    id   => 'DesignPermissions',
    key  => 'designpermissions',
    description => '<__trans phrase="Permissions for Widgets and Style.">',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version => $VERSION,
    l10n_class => 'DesignPermissions::L10N',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        callbacks => {
            'MT::App::CMS::pre_run'
                => {
                    handler => \&_pre_run,
                    priority => 1,
                },
        },
        permissions => {
            'blog.edit_widgets' => {
                label => 'Manage Widgets',
                group => 'blog_design',
                order => 1000,
                permitted_action => { edit_templates => 1 },
            },
            'blog.edit_style' => {
                label => 'Manage Styles',
                group => 'blog_design',
                order => 1001,
                permitted_action => { edit_templates => 1 },
            },
        },
   } );
}

sub _pre_run {
    my $app = MT->instance;
    no warnings 'redefine';
    *StyleCatcher::CMS::js = \&_js;
    if ( my $stylecatcher = MT->component( 'StyleCatcher' ) ) {
        my $r = $stylecatcher->registry( 'applications', 'cms', 'menus' );
        $r->{ 'design:theme' } = { label      => MT->translate( 'Styles' ),
                                   order      => 300,
                                   mode       => 'stylecatcher_theme',
                                   view       => [ 'blog', 'website' ],
                                   permission => 'edit_style', };
    }
    if ( my $core = MT->component( 'core' ) ) {
        my $r = $core->registry( 'applications', 'cms', 'menus' );
        $r->{ 'design:widgets' } = { label             => MT->translate( 'Widgets' ),
                                     order             => 200,
                                     mode              => 'list_widget',
                                     permission        => 'edit_widgets',
                                     system_permission => 'edit_templates',
                                     view              => [ "blog", 'website', 'system' ] };
    }
    if ( $app->blog ) {
        my $blog = $app->blog;
        if ( ( $app->mode eq 'list_widget' ) ||
             ( $app->mode eq 'edit_widget' ) ||
             ( $app->mode eq 'save_widget' ) ) {
            if (! is_user_can( $blog, $app->user, 'edit_widgets' ) ) {
                $app->return_to_dashboard( permission => 1 );
            } else {
                my $perms = $app->permissions->{ column_values }->{ permissions };
                $app->permissions->{ column_values }->{ permissions } = "$perms,'edit_templates'";
            }
        }
        if ( ( $app->mode eq 'stylecatcher_theme' ) ||
             ( $app->mode eq 'stylecatcher_apply' ) ) {
            if (! is_user_can( $blog, $app->user, 'edit_style' ) ) {
                $app->return_to_dashboard( permission => 1 );
            } else {
                my $perms = $app->permissions->{ column_values }->{ permissions };
                $app->permissions->{ column_values }->{ permissions } = "$perms,'edit_templates'";
            }
        }
    }
}

sub is_user_can {
    my ( $blog, $user, $permission ) = @_;
    $permission = 'can_' . $permission;
    my $perm = $user->is_superuser;
    if (! $perm ) {
        if ( $blog ) {
            $perm = $user->permissions( $blog->id )->$permission;
        }
    }
    return $perm;
}

sub _js {
    my $app = shift;
    # return $app->json_error( 'Permission Denied.' )
    #     unless $app->can_do( 'edit_templates' );
    return $app->json_error( $app->errstr ) unless $app->validate_magic;
    require StyleCatcher::CMS;
    my $data = StyleCatcher::CMS::fetch_themes( $app->param( 'url' ) )
        or return $app->json_error( $app->errstr );
    return $app->json_result( $data );
}

1;